(define (runge-kutta-4 slope-fn y0 final-x divisions)
  (next-rk4 slope-fn divisions (/ final-x (+ .0 divisions)) 
	    (/ (/ final-x (+ divisions .0)) 2) 0. (+ y0 .0) 0))

(define (next-rk4 slope-fn divisions step step2 x y div)
  (if (= divisions div)
      (cons x y)
      (let* ((slope-a (slope-fn x y))
	     (slope-b (slope-fn (flo:+ x step2)
				(flo:+ y (flo:* step2 slope-a))))
	     (slope-c (slope-fn (flo:+ x step2)
				(flo:+ y (flo:* step2 slope-b))))
	     (slope-d (slope-fn (flo:+ x step)
				(flo:+ y (flo:* step slope-c))))
	     (slope (flo:/ (+ slope-a
			      (flo:* 2. slope-b)
			      (flo:* 2. slope-c)
			      slope-d)
			   6.)))
	(next-rk4 slope-fn
		  divisions
		  step
		  step2
		  (flo:+ x step)
		  (flo:+ y (flo:* step slope))
		  (+ div 1)))))

(define (riccati x y) (flo:- 1. (flo:* x (flo:* y y))))

(define (critical-y0 function approx-function 
		     start-y0 end-y0 y-step end-x x-step)
  (let* ((start-y0 (+ start-y0 .0))
	 (end-y0 (+ end-y0 .0))
	 (y-step (+ y-step .0))
	 (end-x (+ end-x .0))
	 (x-step (+ x-step .0))
	 (y-incr (flo:/ (flo:- end-y0 start-y0) y-step)))
    (define (critical-helper current-y0)
      (cond ((> current-y0 end-y0) #f)
	    ((< 0 (cdr (approx-function function 
					current-y0 
					end-x 
					x-step))) 
	     current-y0)
	    (else (critical-helper (flo:+ current-y0 y-incr)))))
    (critical-helper start-y0)))

(display
"
The following procedures are now defined:

riccati: a Riccati equation y'=1-xy^2
  arguments: x - a floating point number (gives errors on integers!)
             y - ditto
  returns: slope (floating point)

runge-kutta-4: scheme 4th-order Runge-Kutta implementation
  arguments: function - function to approximate.  The function 
                        must take two floating-point arguments
                        and return the value of the slope at that point
             y0 - the initial y value of the function
             final-x - the final x-value at which to approximate the 
                       value of y
             divisions - the number of divisions to use.  More accuracy
                         comes with more divisions.
  returns: (x-value . y-value) (a single cons cell, not a list!)


critical-y0: find the initial y0 at which a function is positive at a
             certain point
  arguments: function - the function to approximate.  Must have the
                        properties of the function riccati above
             approx-function - the function used to approximate.  Must
                               be similar to runge-kutta-4 in argument
                               and return value
             start-y0 - the initial y0 to test
             end-y0 - the y0 to stop testing with
             y-step - the number of steps to attempt between start and 
                      end
             x-value - the value at which to test against zero
             x-step - the number of steps used by the approx-function
                      in determining the current x value
  returns: critical-y0 (floating point)
"
)    

